@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: cryptokit.ml,v 1.10 2003/07/10 13:37:38 xleroy Exp $ *)
+(* $Id: cryptokit.ml,v 1.17 2005/04/20 12:43:55 xleroy Exp $ *)
 
 open Nat
 
@@ -56,6 +56,12 @@ external arcfour_transform : string -> string -> int -> string -> int -> int ->
 external sha1_init: unit -> string = "caml_sha1_init"
 external sha1_update: string -> string -> int -> int -> unit = "caml_sha1_update"
 external sha1_final: string -> string = "caml_sha1_final"
+external sha256_init: unit -> string = "caml_sha256_init"
+external sha256_update: string -> string -> int -> int -> unit = "caml_sha256_update"
+external sha256_final: string -> string = "caml_sha256_final"
+external ripemd160_init: unit -> string = "caml_ripemd160_init"
+external ripemd160_update: string -> string -> int -> int -> unit = "caml_ripemd160_update"
+external ripemd160_final: string -> string = "caml_ripemd160_final"
 external md5_init: unit -> string = "caml_md5_init"
 external md5_update: string -> string -> int -> int -> unit = "caml_md5_update"
 external md5_final: string -> string = "caml_md5_final"
@@ -73,6 +79,7 @@ class type transform =
     method put_byte: int -> unit
 
     method finish: unit
+    method flush: unit
 
     method available_output: int
 
@@ -142,6 +149,7 @@ class compose (tr1 : transform) (tr2 : transform) =
     method get_char = tr2#get_char
     method get_byte = tr2#get_byte
 
+    method flush = tr1#flush; self#transfer; tr2#flush
     method finish = tr1#finish; self#transfer; tr2#finish
 
     method wipe = tr1#wipe; tr2#wipe
@@ -552,7 +560,7 @@ class cipher (cipher : block_cipher) =
       output_buffer#wipe;
       wipe_string ibuf
 
-    method finish =
+    method flush =
       if used = 0 then ()
       else if used = blocksize then begin
         self#ensure_capacity blocksize;
@@ -561,6 +569,9 @@ class cipher (cipher : block_cipher) =
         oend <- oend + blocksize
       end
       else raise (Error Wrong_data_length)
+
+    method finish =
+      self#flush
   end
 
 (* Block cipher with padding *)
@@ -742,6 +753,7 @@ class cipher (cipher : stream_cipher) =
     method put_byte b =
       self#put_char (Char.unsafe_chr b)
 
+    method flush = ()
     method finish = ()
 
     method wipe =
@@ -778,6 +790,50 @@ class sha1 =
 
 let sha1 () = new sha1
 
+class sha256 =
+  object(self)
+    val context = sha256_init()
+    method hash_size = 32
+    method add_substring src ofs len =
+      if ofs < 0 || ofs + len > String.length src
+      then invalid_arg "sha256#add_substring";
+      sha256_update context src ofs len
+    method add_string src =
+      sha256_update context src 0 (String.length src)
+    method add_char c =
+      self#add_string (String.make 1 c)
+    method add_byte b =
+      self#add_char (Char.unsafe_chr b)
+    method result =
+      sha256_final context
+    method wipe =
+      wipe_string context
+  end
+
+let sha256 () = new sha256
+
+class ripemd160 =
+  object(self)
+    val context = ripemd160_init()
+    method hash_size = 32
+    method add_substring src ofs len =
+      if ofs < 0 || ofs + len > String.length src
+      then invalid_arg "ripemd160#add_substring";
+      ripemd160_update context src ofs len
+    method add_string src =
+      ripemd160_update context src 0 (String.length src)
+    method add_char c =
+      self#add_string (String.make 1 c)
+    method add_byte b =
+      self#add_char (Char.unsafe_chr b)
+    method result =
+      ripemd160_final context
+    method wipe =
+      wipe_string context
+  end
+
+let ripemd160 () = new ripemd160
+
 class md5 =
   object(self)
     val context = md5_init()
@@ -860,7 +916,10 @@ end
 module HMAC(H: sig class h: hash  val blocksize: int end) =
   struct
     let hmac_pad key byte =
-      if String.length key > H.blocksize then raise(Error Wrong_key_size);
+      let key =
+        if String.length key > H.blocksize
+        then hash_string (new H.h) key
+        else key in
       let r = String.make H.blocksize (Char.chr byte) in
       xor_string key 0 r 0 (String.length key);
       r
@@ -1607,11 +1666,14 @@ let new_key ?rng ?e numbits =
         if Bn.relative_prime (Bn.sub n Bn.one) (nat_of_int e)
         then n
         else gen_factor nbits in
-  let p = gen_factor numbits2
-  and q = gen_factor numbits2 in
-  (* Make sure p >= q *)
-  let (p, q) =
-    if Bn.compare p q < 0 then (q, p) else (p, q) in
+  (* Make sure p > q *)
+  let rec gen_factors nbits =
+    let p = gen_factor nbits
+    and q = gen_factor nbits in
+    let cmp = Bn.compare p q in
+    if cmp = 0 then gen_factors nbits else
+    if cmp < 0 then (q, p) else (p, q) in
+  let (p, q) = gen_factors numbits2 in
   (* p1 = p - 1 and q1 = q - 1 *)
   let p1 = Bn.sub p Bn.one
   and q1 = Bn.sub q Bn.one in
@@ -1633,7 +1695,7 @@ let new_key ?rng ?e numbits =
   let d = Bn.mod_inv e (Bn.mult p1 q1) in
   (* dp = d mod p-1 and dq = d mod q-1 *)
   let dp = Bn.mod_ d p1 and dq = Bn.mod_ d q1 in
-  (* qinv = q^1 mod p *)
+  (* qinv = q^-1 mod p *)
   let qinv = Bn.mod_inv q p in
   (* Build key *)
   let res =
@@ -1754,6 +1816,8 @@ class encode multiline =
 
     method put_byte b = self#put_char (Char.chr b)
 
+    method flush : unit = raise (Error Wrong_data_length)
+
     method finish =
       begin match ipos with
         1 ->
@@ -1840,6 +1904,8 @@ class decode =
 
     method put_byte b = self#put_char (Char.chr b)
 
+    method flush : unit = raise (Error Wrong_data_length)
+
     method finish =
       finished <- true;
       match ipos with
@@ -1890,6 +1956,7 @@ class encode =
     method put_string s =
       self#put_substring s 0 (String.length s)
 
+    method flush = ()
     method finish = ()
 
     method wipe = output_buffer#wipe
@@ -1936,6 +2003,9 @@ class decode =
 
     method put_byte b = self#put_char (Char.chr b)
 
+    method flush =
+      if ipos <> 0 then raise(Error Wrong_data_length)
+
     method finish =
       if ipos <> 0 then raise(Error Bad_encoding)
 
@@ -1977,14 +2047,14 @@ class compress level =
   object(self)
     val zs = deflate_init level false
     
-    inherit buffered_output 256 as output_buffer
+    inherit buffered_output 512 as output_buffer
 
     method input_block_size = 1
     method output_block_size = 1
 
     method put_substring src ofs len =
       if len > 0 then begin
-        self#ensure_capacity 64;
+        self#ensure_capacity 256;
         let (_, used_in, used_out) =
           deflate zs
                   src ofs len
@@ -2001,8 +2071,19 @@ class compress level =
 
     method put_byte b = self#put_char (Char.chr b)
 
+    method flush =
+      self#ensure_capacity 256;
+      let avail_out = String.length obuf - oend in
+      let (_, _, used_out) =
+         deflate zs
+                 "" 0 0
+                 obuf oend (String.length obuf - oend)
+                 Z_SYNC_FLUSH in
+      oend <- oend + used_out;
+      if oend = String.length obuf then self#flush
+
     method finish =
-      self#ensure_capacity 64;
+      self#ensure_capacity 256;
       let (finished, _, used_out) =
          deflate zs
                  "" 0 0
@@ -2021,22 +2102,26 @@ class uncompress =
   object(self)
     val zs = inflate_init false
     
-    inherit buffered_output 256 as output_buffer
+    inherit buffered_output 512 as output_buffer
 
     method input_block_size = 1
     method output_block_size = 1
 
     method put_substring src ofs len =
       if len > 0 then begin
-        self#ensure_capacity 64;
-        let (_, used_in, used_out) =
+        self#ensure_capacity 256;
+        let (finished, used_in, used_out) =
           inflate zs
                   src ofs len
                   obuf oend (String.length obuf - oend)
                   Z_SYNC_FLUSH in
         oend <- oend + used_out;
-        if used_in < len
-        then self#put_substring src (ofs + used_in) (len - used_in)
+        if used_in < len then begin
+          if finished then
+            raise(Error(Compression_error("Zlib.uncompress",
+               "garbage at end of compressed data")));
+          self#put_substring src (ofs + used_in) (len - used_in)
+        end
       end
 
     method put_string s = self#put_substring s 0 (String.length s)
@@ -2045,9 +2130,11 @@ class uncompress =
 
     method put_byte b = self#put_char (Char.chr b)
 
+    method flush = ()
+
     method finish =
       let rec do_finish first_finish =
-        self#ensure_capacity 64;
+        self#ensure_capacity 256;
         let (finished, _, used_out) =
            inflate zs
                    " " 0 (if first_finish then 1 else 0)
