@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: cryptokit.mli,v 1.13 2003/07/10 13:37:38 xleroy Exp $ *)
+(* $Id: cryptokit.mli,v 1.20 2005/04/20 12:44:11 xleroy Exp $ *)
 
 (** The Cryptokit library provides a variety of cryptographic primitives
     that can be used to implement cryptographic protocols in
@@ -57,14 +57,28 @@ class type transform =
     method finish: unit
       (** Call method [finish] to indicate that no further data will
           be processed through the transform.  This causes the transform
-          to e.g. add final padding to the data and flush its internal
-          buffers.  Raise [Error Wrong_data_length] if the total length
+          to flush its internal buffers and perform all appropriate
+          finalization actions, e.g. add final padding.  Raise [Error
+          Wrong_data_length] if the total length of input data
+          provided via the [put_*] methods is not an integral number
+          of the input block size (see
+          {!Cryptokit.transform.input_block_size}).  After calling
+          [finish], the transform can no longer accept additional
+          data.  Hence, do not call any of the [put_*] methods nor
+          [flush] after calling [finish]. *)
+    method flush: unit
+      (** [flush] causes the transform to flush its internal buffers
+          and make all output processed up to this point available through
+          the [get_*] methods.  
+          Raise [Error Wrong_data_length] if the total length
           of input data provided via the [put_*] methods is not
           an integral number of the input block size
           (see {!Cryptokit.transform.input_block_size}).
-          After calling [finish], the transform can no longer accept
-          additional data.  Hence, do not call any of the [put_*]
-          methods after calling [finish]. *)
+          (For padded block ciphers, the input block size used here
+          is that of the underlying block cipher, without the padding.)
+          Unlike method [finish], method [flush] does not add final
+          padding and leaves the transform in a state where it can
+          still accept more input. *)
     method available_output: int
       (** Return the number of characters of output currently available.
           The output can be recovered with the [get_*] methods. *)
@@ -105,7 +119,7 @@ class type transform =
       (** Some transforms (e.g. block ciphers) always produce output
           data by blocks of several characters.  This method
           returns the size of output blocks for the current transform.
-          If [input_block_size > 1], the total length of output data
+          If [output_block_size > 1], the total length of output data
           produced by the transform is always an integral multiple
           of [output_block_size].
           If [output_block_size = 1], the transform produces output data
@@ -116,7 +130,7 @@ class type transform =
           data such as secret key-derived material, or parts of the
           input or output data.  Calling [wipe] ensures that this sensitive
           data will not remain in memory longer than strictly necessary,
-          thus making certain invasive attacks more difficult.
+          thus making invasive attacks more difficult.
           It is thus prudent practice to call [wipe] on every
           transform that the program no longer needs.
           After calling [wipe], the transform is no longer in a working
@@ -229,7 +243,7 @@ module Random : sig
         if no suitable RNG is available.
         [secure_rng#random_bytes] may block until enough entropy
         has been gathered.  Do not use for generating large quantities
-        of random data, else you might exhaust the entropy sources
+        of random data, otherwise you could exhaust the entropy sources
         of the system. *)
 
   val system_rng: unit -> rng
@@ -423,8 +437,8 @@ module Cipher : sig
         function as encryption. *)
 end
 
-(** The [Hash] module implements unkeyed cryptographic hashes SHA-1
-    and MD5, also known as message digest functions.  
+(** The [Hash] module implements unkeyed cryptographic hashes (SHA-1,
+    SHA-256, RIPEMD-160 and MD5), also known as message digest functions.  
     Hash functions used in cryptography are characterized as being
     <I>one-way</I> (given a hash value, it is computationally
     infeasible to find a text that hashes to this value) and
@@ -435,12 +449,18 @@ end
 module Hash : sig
   val sha1: unit -> hash
     (** SHA-1 is the Secure Hash Algorithm revision 1.  It is a NIST
-        standard, is widely used, and has no known weaknesses.
-        It produces 160-bit hashes (20 bytes).  *)
+        standard, is widely used, and produces 160-bit hashes (20 bytes).
+        Recent results suggest that it is not collision-resistant. *)
+  val sha256: unit -> hash
+    (** SHA-256, another NIST standard, is a variant of SHA-1 that
+        produces 256-bit hashes (32 bytes). *)
+  val ripemd160: unit -> hash
+    (** RIPEMD-160 produces 160-bit hashes (20 bytes). *)
   val md5: unit -> hash
     (** MD5 is an older hash function, producing 128-bit hashes (16 bytes).
-        While popular in many applications, it is considered as
-        potentially slightly weaker than SHA-1. *)
+        While popular in many legacy applications, it is now
+        considered as unsecure.  In particular, it is not
+        collision-resistant. *)
 end
 
 (** The [MAC] module implements message authentication codes, also
@@ -461,15 +481,13 @@ module MAC: sig
   val hmac_sha1: string -> hash
     (** [hmac_sha1 key] returns a MAC based on the HMAC construction (RFC2104)
         applied to SHA-1.  The returned hash values are 160 bits (20 bytes)
-        long.  The [key] argument is the MAC key; it can have any length
-        between 1 and 64 bytes.  A minimal length of 20 bytes is
-        recommended. *)
+        long.  The [key] argument is the MAC key; it can have any length,
+        but a minimal length of 20 bytes is recommended. *)
   val hmac_md5: string -> hash
     (** [hmac_md5 key] returns a MAC based on the HMAC construction (RFC2104)
         applied to MD5.  The returned hash values are 128 bits (16 bytes)
-        long.  The [key] argument is the MAC key; it can have any length
-        between 1 and 64 bytes.  A minimal length of 16 bytes is
-        recommended. *)
+        long.  The [key] argument is the MAC key; it can have any length,
+        but a minimal length of 16 bytes is recommended. *)
   val aes: ?iv:string -> ?pad:Padding.scheme -> string -> hash
     (** [aes key] returns a MAC based on AES encryption in CBC mode.
         The ciphertext is discarded, and the final value of the
@@ -532,10 +550,10 @@ module RSA: sig
     }
     (** The type of RSA keys.  Components [size], [n] and [e] define
         the public part of the key.  Components [size], [n] and [d]
-        define the private part of the key.  To speed up secret key operations
+        define the private part of the key.  To speed up private key operations
         through the use of the Chinese remainder theorem (CRT), additional
         components [p], [q], [dp], [dq] and [qinv] are provided.  These
-        are part of the secret key. *)
+        are part of the private key. *)
 
   val wipe_key: key -> unit
     (** Erase all components of a RSA key. *)
