@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: stubs-md5.c,v 1.2 2003/07/10 14:20:31 xleroy Exp $ */
+/* $Id: stubs-md5.c,v 1.3 2004/08/20 12:05:55 xleroy Exp $ */
 
 #include <caml/mlvalues.h>
 #include <caml/memory.h>
@@ -23,23 +23,23 @@ struct MD5Context {
         unsigned char in[64];
 };
 
-CAMLextern void MD5Init (struct MD5Context *context);
-CAMLextern void MD5Update (struct MD5Context *context,
+CAMLextern void caml_MD5Init (struct MD5Context *context);
+CAMLextern void caml_MD5Update (struct MD5Context *context,
                            unsigned char *buf, unsigned len);
-CAMLextern void MD5Final (unsigned char *digest, struct MD5Context *ctx);
+CAMLextern void caml_MD5Final (unsigned char *digest, struct MD5Context *ctx);
 
 #define Context_val(v) ((struct MD5Context *) String_val(v))
 
 CAMLprim value caml_md5_init(value unit)
 {
   value ctx = alloc_string(sizeof(struct MD5Context));
-  MD5Init(Context_val(ctx));
+  caml_MD5Init(Context_val(ctx));
   return ctx;
 }
 
 CAMLprim value caml_md5_update(value ctx, value src, value ofs, value len)
 {
-  MD5Update(Context_val(ctx), &Byte_u(src, Long_val(ofs)), Long_val(len));
+  caml_MD5Update(Context_val(ctx), &Byte_u(src, Long_val(ofs)), Long_val(len));
   return Val_unit;
 }
 
@@ -49,7 +49,7 @@ CAMLprim value caml_md5_final(value ctx)
   CAMLlocal1(res);
 
   res = alloc_string(16);
-  MD5Final(&Byte_u(res, 0), Context_val(ctx));
+  caml_MD5Final(&Byte_u(res, 0), Context_val(ctx));
   CAMLreturn(res);
 }
 
