@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: test.ml,v 1.6 2003/07/10 13:37:38 xleroy Exp $ *)
+(* $Id: test.ml,v 1.9 2005/02/22 15:04:37 xleroy Exp $ *)
 
 (* Test vectors *)
 
@@ -165,6 +165,40 @@ let _ =
   test 6 (hash (String.make 1000000 'a'))
          (hex "34AA973CD4C4DAA4F61EEB2BDBAD27316534016F")
 
+(* SHA-256 *)
+let _ =
+  testing_function "SHA-256";
+  let hash s = hash_string (Hash.sha256()) s in
+  test 1 (hash "abc")
+    (hex "ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad");
+  test 2 (hash "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
+    (hex "248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1");
+  test 3 (hash (String.make 1000000 'a'))
+    (hex "cdc76e5c9914fb9281a1c7e284d73e67f1809a48a497200e046d39ccc7112cd0")
+
+(* RIPEMD-160 *)
+let _ =
+  testing_function "RIPEMD-160";
+  let hash s = hash_string (Hash.ripemd160()) s in
+  test 1 (hash "")
+    (hex "9c1185a5c5e9fc54612808977ee8f548b2258d31");
+  test 2 (hash "a")
+    (hex "0bdc9d2d256b3ee9daae347be6f4dc835a467ffe");
+  test 3 (hash "abc")
+    (hex "8eb208f7e05d987a9b044a8e98c6b087f15a0bfc");
+  test 4 (hash "message digest")
+    (hex "5d0689ef49d2fae572b881b123a85ffa21595f36");
+  test 5 (hash "abcdefghijklmnopqrstuvwxyz")
+    (hex "f71c27109c692c1b56bbdceb5b9d2865b3708dbc");
+  test 6 (hash "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
+    (hex "12a053384a9c0c88e405a06c27dcf49ada62eb2b");
+  test 7 (hash "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
+    (hex "b0e20b6e3116640286ed3a87a5713079b21f5189");
+  test 8 (hash "12345678901234567890123456789012345678901234567890123456789012345678901234567890")
+    (hex "9b752e45573d4b39f4dbd3323cab82bf63326bfb");
+  test 9 (hash (String.make 1000000 'a'))
+    (hex "52783243c1697bdbe16d37f97f68f08325dc1528")
+
 (* MD5 *)
 let _ =
   testing_function "MD5";
@@ -411,7 +445,12 @@ The quick brown fox jumps over the lazy dog.
 The quick brown fox jumps over the lazy dog.
 " in
   try
-    test 1 text (transform_string (Zlib.uncompress()) (transform_string (Zlib.compress()) text))
+    test 1 text (transform_string (Zlib.uncompress()) (transform_string (Zlib.compress()) text));
+    let c = Zlib.compress() and u = Zlib.uncompress() in
+    c#put_string text; c#flush; u#put_string c#get_string; u#flush;
+    test 2 text u#get_string;
+    c#put_string text; c#finish; u#put_string c#get_string; u#finish;
+    test 3 text u#get_string
   with Error Compression_not_supported ->
     printf " (not supported)"
 
